%% Import data
Lac = readtable("Lactate.xlsx");


%% Plot raw data
W = Lac.WorkLoad_W_;
L = Lac.Lactate_mmol_L_;

fig1 = figure;
plot(W(1:end-2),L(1:end-2),'bo');
xlabel("Work rate [W]");
ylabel("Blood lactate concentration [mmol/L]");
title("Raw data");

saveas(fig1,"Raw data.png");


%% Aerobic lactate threshold (The first increase of > 0.5 mmol / L)
for i = 1:length(L)-1
    if L(i+1) - L(i) > 0.5
        I_LT = i;
        break
    end
end


%% Log log method
figure;
plot(log10(W(1:end-2)),log10(L(1:end-2)),'bo');
xlabel("Log work rate [W]");
ylabel("Log blood lactate concentration [mmol/L]");
title("Log log method");


% Create model and predict
Model_log1 = fitlm(log10(W(2:5)),log10(L(2:5))); % The first value is -inf
Vec_log1 = 1.6:0.1:2.4;
Pred_log1 = predict(Model_log1,Vec_log1');

Model_log2 = fitlm(log10(W(6:end-2)),log10(L(6:end-2)));
Vec_log2 = 2:0.1:2.4;
Pred_log2 = predict(Model_log2,Vec_log2');


% System of linear equation
LT_log_work = (Model_log1.Coefficients.Estimate(1) - Model_log2.Coefficients.Estimate(1))...
    /(Model_log2.Coefficients.Estimate(2) - Model_log1.Coefficients.Estimate(2));

Pred_log_LT = predict(Model_log1,LT_log_work);


% Convert log to original value
LT_work = 10^LT_log_work;

Pred_LT = 10^Pred_log_LT;


% Check prediction on log data plot
fig2 = figure;
plot(log10(W(1:end-2)),log10(L(1:end-2)),'bo'),hold on;
plot(Vec_log1,Pred_log1,'k-');
plot(Vec_log2,Pred_log2,"k-");
plot(LT_log_work,Pred_log_LT,"go");
xlabel("Log work rate [W]");
ylabel("Log blood lactate concentration [mmol/L]");
title("Log log method");

saveas(fig2,"Log log method.png");

%Check prediction on raw data plot
figure;
plot(W(1:end-2),L(1:end-2),'bo'),hold on;
plot(LT_work,Pred_LT,'ro');
xlabel("Work rate [W]");
ylabel("Blood lactate concentration [mmol/L]");
title("Raw data");


%% Fitting
% Exponential fitting
[F_exp, gof_exp] = fit(W(1:end-2),L(1:end-2),'exp1');
figure;
plot(F_exp,W(1:end-2),L(1:end-2));
legend("Location","best");
xlabel("Work rate [W]");
ylabel("Blood lactate concentration [mmol / L]");
title("Exponential fitting");

% Double exponential fitting
[F_exp2, gof_exp2] = fit(W(1:end-2),L(1:end-2),'exp2');
figure;
plot(F_exp2,W(1:end-2),L(1:end-2));
legend("Location","best");
xlabel("Work rate [W]");
ylabel("Blood lactate concentration [mmol / L]");
title("Double exponential fitting");

% Polynomial fitting
[F_poly, gof_poly] = fit(W(1:end-2),L(1:end-2),'poly2');
figure;
plot(F_poly,W(1:end-2),L(1:end-2));
legend("Location","best");
xlabel("Work rate [W]");
ylabel("Blood lactate concentration [mmol / L]");
title("Polynomial fitting");

% Interpolation fitting
[F_interp, gof_interp] = fit(W(1:end-2),L(1:end-2),'cubicinterp');
figure;
plot(F_interp,W(1:end-2),L(1:end-2));
legend("Location","best");
xlabel("Work rate [W]");
ylabel("Blood lactate concentration [mmol / L]");
title("Interpolation fitting");

% Smooth spline fitting
[F_spline, gof_spline] = fit(W(1:end-2),L(1:end-2),'smoothingspline');
figure;
plot(F_spline,W(1:end-2),L(1:end-2));
legend("Location","best");
xlabel("Work rate [W]");
ylabel("Blood lactate concentration [mmol / L]");
title("Spline fitting");



% Plot all
fig3 = figure;
subplot(3,2,1);
plot(F_exp,W(1:end-2),L(1:end-2));
title("Exponential");
legend("Location","best");
subplot(3,2,2);
plot(F_exp2,W(1:end-2),L(1:end-2));
title("double exponential");
legend("Location","best");
subplot(3,2,3);
plot(F_poly,W(1:end-2),L(1:end-2));
title("Quadric polynomial");
legend("Location","best");
subplot(3,2,4);
plot(F_interp,W(1:end-2),L(1:end-2));
title("Cubic interpolation");
legend("Location","best");
subplot(3,2,5);
plot(F_spline,W(1:end-2),L(1:end-2));
title("Smooth spline");
legend("Location","best");

saveas(fig3,"Fitting.png");


%% Mader (LT = 4mmol/L)
x = W(1):W(end-2);
Fitted_exp2 = F_exp2.a*exp(F_exp2.b.*x) + F_exp2.c*exp(F_exp2.d.*x);
I_Mader = find(Fitted_exp2 >= 4,1);
x_Mader = x(I_Mader);
y_Mader = Fitted_exp2(I_Mader);

fig4 = figure;
plot(W(1:end-2),L(1:end-2),'bo'),hold on;
plot(Fitted_exp2);
plot(x_Mader,y_Mader,"g*");
yline(4);
xlabel("Work rate [W]");
ylabel("Blood lactate concentration [mmol/L]");
title("Mader");
legend(["Raw data" "Fitted curve" "LT"],"Location","best");

saveas(fig4,"Mader.png");


%% Dickhut (LT = Minimum blood lactate + 1.5mmol / L)
x = W(1):W(end-2);
Fitted_exp2 = F_exp2.a*exp(F_exp2.b.*x) + F_exp2.c*exp(F_exp2.d.*x);
Min_lac = min(L);
I_Dickhut = find(Fitted_exp2 >= Min_lac+1.5,1);
x_Dickhut = x(I_Dickhut);
y_Dickhut = Fitted_exp2(I_Dickhut);

fig5 = figure;
plot(W(1:end-2),L(1:end-2),'bo'),hold on;
plot(Fitted_exp2);
plot(x_Dickhut,y_Dickhut,"g*");
yline(Min_lac,"-","Base");
yline(y_Dickhut,"-","Base + 1.5");
xlabel("Work rate [W]");
ylabel("Blood lactate concentration [mmol/L]");
title("Dickhut");
legend(["Raw data" "Fitted curve" "LT"],"Location","best");
ylim([-1 8]);

saveas(fig5, "Dickhut.png");


%% Freiburger (LT = Aerobic LT + 1.5mmol / L)
x = W(1):W(end-2);
Fitted_exp2 = F_exp2.a*exp(F_exp2.b.*x) + F_exp2.c*exp(F_exp2.d.*x);
LT = L(I_LT);
I_Freiburger = find(Fitted_exp2 >= LT+1.5,1);
x_Freiburger = x(I_Freiburger);
y_Freiburger = Fitted_exp2(I_Freiburger);

fig6 = figure;
plot(W(1:end-2),L(1:end-2),'bo'),hold on;
plot(Fitted_exp2);
plot(x_Freiburger,y_Freiburger,"g*");
yline(LT,"-","Aerobic LT");
yline(y_Freiburger,"-","Aerobic LT + 1.5");
xlabel("Work rate [W]");
ylabel("Blood lactate concentration [mmol/L]");
title("Freiburger");
legend(["Raw data" "Fitted curve" "LT"],"Location","best");

saveas(fig6,"Freiburger.png");


%% Dmax (LT = The biggest distance btw a line from the first to last data point and lactate curve)
x = W(1):W(end-2);

Fitted_exp2 = F_exp2.a*exp(F_exp2.b.*x) + F_exp2.c*exp(F_exp2.d.*x);

Slope_Dmax = (Fitted_exp2(end)-Fitted_exp2(1))/(W(end-2)-W(1));

Y_intersection_Dmax = Fitted_exp2(end)-Slope_Dmax*W(end-2); % y = slope*x+b, so b = fitted value-slope*W

Linear_Dmax = Slope_Dmax*x+Y_intersection_Dmax;
x_Dmax = log((Slope_Dmax - F_exp2.a*F_exp2.b *exp(-(F_exp2.b...
    *log((F_exp2.a * F_exp2.b)/(F_exp2.c * F_exp2.d)))...
    /(F_exp2.b - F_exp2.d)))/(F_exp2.c * F_exp2.d))/F_exp2.d;
x_Dmax = real(x_Dmax);

y_Dmax = F_exp2.a * exp(x_Dmax * F_exp2.b) + F_exp2.c * exp(x_Dmax * F_exp2.d);

fig7 = figure;
plot(W(1:end-2),L(1:end-2),'bo'),hold on;
plot(Fitted_exp2);
plot(x_Dmax,y_Dmax,"g*");
plot(x,Linear_Dmax,"m-");
xlabel("Work rate [W]");
ylabel("Blood lactate concentration [mmol/L]");
title("Dmax");
legend(["Raw data" "Fitted curve" "LT" "Linear fit"],"Location","best");

saveas(fig7,"Dmax.png");


%% Modified Dmax (LT = The biggest distance btw a line from LT to last data point and lactate curve)
x = W(1):W(end-2);

Fitted_exp2 = F_exp2.a*exp(F_exp2.b.*x) + F_exp2.c*exp(F_exp2.d.*x);

I_fitted_LT = find(Fitted_exp2 >= L(I_LT),1);

Slope_Mod_Dmax = (Fitted_exp2(end)-Fitted_exp2(I_fitted_LT))/(W(end-2)-W(I_LT));

Y_intersection_Mod_Dmax = Fitted_exp2(end)-Slope_Mod_Dmax*W(end-2); % y = slope*x+b, so b = fitted value-slope*W

Linear_Mod_Dmax = Slope_Mod_Dmax*x+Y_intersection_Mod_Dmax;

x_Mod_Dmax = log((Slope_Mod_Dmax - F_exp2.a*F_exp2.b *exp(-(F_exp2.b...
    *log((F_exp2.a * F_exp2.b)/(F_exp2.c * F_exp2.d)))...
    /(F_exp2.b - F_exp2.d)))/(F_exp2.c * F_exp2.d))/F_exp2.d;
x_Mod_Dmax = real(x_Mod_Dmax);

y_Mod_Dmax = F_exp2.a * exp(x_Mod_Dmax * F_exp2.b) + F_exp2.c * exp(x_Mod_Dmax * F_exp2.d);

fig8 = figure;
plot(W(1:end-2),L(1:end-2),'bo'),hold on;
plot(Fitted_exp2);
plot(x_Mod_Dmax,y_Mod_Dmax,"g*");
plot(x,Linear_Mod_Dmax,"m-");
xlabel("Work rate [W]");
ylabel("Blood lactate concentration [mmol/L]");
title("Modified Dmax");
legend(["Raw data" "Fitted curve" "LT" "Linear fit"],"Location","best");

saveas(fig8,"Modified Dmax.png");


%% Kindermann (LT = The sharpest linear line from recovery lactate to a fitted value)
W_kinder = W;
W_kinder(end-1:end,1) = [264; 288];


% Exponential fitting
[F_kinder_exp, gof_kinder_exp] = fit(W_kinder,L,'exp1');
figure;
plot(F_kinder_exp,W_kinder,L);
legend("Location","best");
xlabel("Work rate [W]");
ylabel("Blood lactate concentration [mmol / L]");
title("Exponential fitting");

% Double exponential fitting
[F_kinder_exp2, gof_kinder_exp2] = fit(W_kinder,L,'exp2');
figure;
plot(F_kinder_exp2,W_kinder,L);
legend("Location","best");
xlabel("Work rate [W]");
ylabel("Blood lactate concentration [mmol / L]");
title("Double exponential fitting");

% Polynomial fitting
[F_kinder_poly, gof_kinder_poly] = fit(W_kinder,L,'poly2');
figure;
plot(F_kinder_poly,W_kinder,L);
legend("Location","best");
xlabel("Work rate [W]");
ylabel("Blood lactate concentration [mmol / L]");
title("Polynomial fitting");

% Smooth spline fitting
[F_kinder_spline, gof_kinder_spline] = fit(W_kinder,L,'smoothingspline');
figure;
plot(F_kinder_spline,W_kinder,L);
legend("Location","best");
xlabel("Work rate [W]");
ylabel("Blood lactate concentration [mmol / L]");
title("Spline fitting");

% Fitted value
x_kinder = W_kinder(1):W_kinder(end);
Fitted_kinder_exp2 = F_kinder_exp2.a*exp(F_kinder_exp2.b.*x_kinder) + ...
    F_kinder_exp2.c*exp(F_kinder_exp2.d.*x_kinder);

Slope_kinder = (Fitted_kinder_exp2(end) - Fitted_kinder_exp2)./...
    (x_kinder(end) - x_kinder);

[~,I_kinder] = max(Slope_kinder);

Y_intersection_kinder = Fitted_kinder_exp2(end) - Slope_kinder(I_kinder) * x_kinder(end);

Linear_kinder = Slope_kinder(I_kinder)*x_kinder+Y_intersection_kinder;

fig9 = figure;
plot(F_kinder_exp2,W_kinder,L),hold on;
plot(x_kinder,Linear_kinder);
plot(x_kinder(I_kinder),Fitted_kinder_exp2(I_kinder),'g*');
legend(["Data" "Fitted" "Linear" "LT"],"Location","best");
xlabel("Work rate [W]");
ylabel("Blood lactate concentration [mmol / L]");
title("Kindermann");

saveas(fig9, "Kindermann.png");


%% Plot all
fig10 = figure;
plot(Fitted_exp2,'b-'),hold on;
plot(x_Mader,y_Mader,"r*");
plot(x_Dickhut,y_Dickhut,"g*");
plot(x_Freiburger,y_Freiburger,"k*");
plot(x_Dmax,y_Dmax,"m*");
plot(x_Mod_Dmax,y_Mod_Dmax,"y*");
plot(x_kinder(I_kinder),Fitted_kinder_exp2(I_kinder),'c*');
legend(["Fitted" "Mader" "Dickhut" "Freiburger" "Dmax" "Modified Dmax"...
    "Kindermann"],"Location","best");
xlabel("Work rate [W]");
ylabel("Blood lactate concentration [mmol / L]");

saveas(fig10,"All LT.png");


%% Create table
tab_name = ["W Mader" "Lac Mader" "W Dickhut" "Lac Dickhut" "W Freiburger" ...
    "Lac Freiburger" "W Dmax" "Lac Dmax" "W Modified Dmax" "Lac Modified Dmax" ...
    "W Kindermann" "Lac Kindermann"];
Tab_results = table(x_Mader,y_Mader,x_Dickhut,y_Dickhut,x_Freiburger,y_Freiburger...
    ,x_Dmax,y_Dmax,x_Mod_Dmax,y_Mod_Dmax,x_kinder(I_kinder),...
    Fitted_kinder_exp2(I_kinder),'VariableNames',tab_name);

writetable(Tab_results,"LT results.xlsx");







